@@ -208,6 +208,8 @@ glade_init_check (void)
   if (check_initialised)
     return;
 
+  glade_init_debug_flags ();
+
   /* Make sure path accessors work on osx */
   build_package_paths ();
 
@@ -407,13 +409,9 @@ glade_app_event_handler (GdkEvent *event, gpointer data)
 static void
 glade_app_class_init (GladeAppClass * klass)
 {
-	GObjectClass *object_class;
+  GObjectClass *object_class;
 	
-	object_class = G_OBJECT_CLASS (klass);
-	
-	object_class->constructor  = glade_app_constructor;
-	object_class->dispose      = glade_app_dispose;
-	object_class->finalize     = glade_app_finalize;
+  object_class = G_OBJECT_CLASS (klass);
 
   object_class->constructor = glade_app_constructor;
   object_class->dispose = glade_app_dispose;
