@@ -36,8 +36,10 @@ G_BEGIN_DECLS
 #define GLADE_IS_APP_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GLADE_TYPE_APP))
 #define GLADE_APP_GET_CLASS(o)    (G_TYPE_INSTANCE_GET_CLASS ((o), GLADE_APP, GladeAppClass))
 
-#define GLADE_ENV_CATALOG_PATH    "GLADE_CATALOG_SEARCH_PATH"
-#define GLADE_ENV_MODULE_PATH     "GLADE_MODULE_SEARCH_PATH"
+#define GLADE_ENV_CATALOG_PATH     "GLADE_CATALOG_SEARCH_PATH"
+#define GLADE_ENV_MODULE_PATH      "GLADE_MODULE_SEARCH_PATH"
+#define GLADE_ENV_TESTING          "GLADE_TESTING"
+#define GLADE_ENV_PIXMAP_DIR       "GLADE_PIXMAP_DIR"
 
 typedef struct _GladeApp         GladeApp;
 typedef struct _GladeAppPrivate  GladeAppPrivate;
