@@ -2,7 +2,7 @@
  * glade-design-layout.c
  *
  * Copyright (C) 2006-2007 Vincent Geddes
- *                    2011 Juan Pablo Ugarte
+ *               2011-2013 Juan Pablo Ugarte
  *
  * Authors:
  *   Vincent Geddes <vgeddes@gnome.org>
@@ -73,6 +73,7 @@ typedef enum
 
 struct _GladeDesignLayoutPrivate
 {
+  GladeWidget *gchild;
   GdkWindow *window, *offscreen_window;
 
   gint child_offset;
@@ -80,7 +81,7 @@ struct _GladeDesignLayoutPrivate
   GdkCursor *cursor;            /* Current cursor */
   GdkCursor *cursors[N_ACTIVITY];
 
-  gint current_width, current_height;
+  GdkRectangle child_rect;
   PangoLayout *widget_name;
   gint layout_width;
 
@@ -109,8 +110,8 @@ struct _GladeDesignLayoutPrivate
 
   /* Drag & Drop */
   GtkWidget *drag_source;
-  GtkWidget *drag_icon;
   gint drag_x, drag_y;
+  GladeWidget *drag_dest;
 
   /* Properties */
   GladeDesignView *view;
@@ -128,16 +129,17 @@ G_DEFINE_TYPE (GladeDesignLayout, glade_design_layout, GTK_TYPE_BIN)
 #define RECTANGLE_POINT_IN(rect,x,y) (x >= rect.x && x <= (rect.x + rect.width) && y >= rect.y && y <= (rect.y + rect.height))
 
 static Margins
-gdl_get_margins_from_pointer (GtkWidget *child, GtkWidget *widget, gint x, gint y)
+gdl_get_margins_from_pointer (GladeDesignLayout *layout, GtkWidget *widget, gint x, gint y)
 {
+  GladeDesignLayoutPrivate *priv = GLADE_DESIGN_LAYOUT_PRIVATE (layout);
   gint width, height, xx, yy, top, bottom, left, right;
+  GdkRectangle rec, child_rec;
   Margins margin = 0;
-  GdkRectangle rec;
-  
+
   width = gtk_widget_get_allocated_width (widget);
   height = gtk_widget_get_allocated_height (widget);
 
-  gtk_widget_translate_coordinates (widget, child, 0, 0, &xx, &yy);
+  gtk_widget_translate_coordinates (widget, GTK_WIDGET (layout), 0, 0, &xx, &yy);
   
   top = gtk_widget_get_margin_top (widget);
   bottom = gtk_widget_get_margin_bottom (widget);
@@ -149,6 +151,14 @@ gdl_get_margins_from_pointer (GtkWidget *child, GtkWidget *widget, gint x, gint
   rec.width = width + left + right + (OUTLINE_WIDTH * 2);
   rec.height = height + top + bottom + (OUTLINE_WIDTH * 2);
 
+  gtk_widget_get_allocation (gtk_bin_get_child (GTK_BIN (layout)), &child_rec);
+  child_rec.x = (child_rec.x + priv->child_offset) - OUTLINE_WIDTH;
+  child_rec.y = (child_rec.y + priv->child_offset) - OUTLINE_WIDTH;
+  child_rec.width += OUTLINE_WIDTH * 2;
+  child_rec.height += OUTLINE_WIDTH * 2;
+
+  gdk_rectangle_intersect (&rec, &child_rec, &rec);
+
   if (RECTANGLE_POINT_IN (rec, x, y))
     {      
       if (y <= yy + OUTLINE_WIDTH) margin |= MARGIN_TOP;
@@ -168,9 +178,7 @@ gdl_get_activity_from_pointer (GladeDesignLayout *layout, gint x, gint y)
   
   if (priv->selection)
     {
-      priv->margin = gdl_get_margins_from_pointer (GTK_WIDGET (layout),
-                                                   priv->selection,
-                                                   x, y);
+      priv->margin = gdl_get_margins_from_pointer (layout, priv->selection, x, y);
       
       if (priv->margin)
         {
@@ -374,9 +382,9 @@ glade_design_layout_motion_notify_event (GtkWidget *widget, GdkEventMotion *ev)
       static GtkTargetList *target = NULL;
 
       if (target == NULL)
-        target = gtk_target_list_new (_glade_design_layout_get_dnd_target (), 1);
+        target = gtk_target_list_new (_glade_dnd_get_target (), 1);
 
-      gtk_drag_begin (widget, target, GDK_ACTION_COPY, 1, (GdkEvent*)ev);
+      gtk_drag_begin_with_coordinates (widget, target, 0, 1, (GdkEvent*)ev, x, y);
       return TRUE;
     }
 
@@ -477,49 +485,6 @@ glade_design_layout_motion_notify_event (GtkWidget *widget, GdkEventMotion *ev)
   return FALSE;
 }
 
-typedef struct
-{
-  GtkWidget *toplevel;
-  gint x;
-  gint y;
-  GtkWidget *placeholder;
-  GladeWidget *gwidget;
-} GladeFindInContainerData;
-
-static void
-glade_design_layout_find_inside_container (GtkWidget                *widget,
-                                           GladeFindInContainerData *data)
-{
-  gint x, y, w, h;
-
-  if (data->gwidget || !gtk_widget_get_mapped (widget))
-    return;
-
-  gtk_widget_translate_coordinates (data->toplevel, widget, data->x, data->y,
-                                    &x, &y);
-  
-  /* Margins are not part of the widget allocation */
-  w = gtk_widget_get_allocated_width (widget) + gtk_widget_get_margin_right (widget);
-  h = gtk_widget_get_allocated_height (widget) + gtk_widget_get_margin_bottom (widget);
-
-  if (x >= (0 - gtk_widget_get_margin_left (widget)) && x < w &&
-      y >= (0 - gtk_widget_get_margin_top (widget)) && y < h)
-    {
-      if (GLADE_IS_PLACEHOLDER (widget))
-        data->placeholder = widget;
-      else
-        {
-          if (GTK_IS_CONTAINER (widget))
-            gtk_container_forall (GTK_CONTAINER (widget), (GtkCallback)
-                                  glade_design_layout_find_inside_container,
-                                  data);
-
-          if (!data->gwidget)
-            data->gwidget = glade_widget_get_from_gobject (widget);
-        }
-    }
-}
-
 static gboolean
 glade_project_is_toplevel_active (GladeProject *project, GtkWidget *toplevel)
 {
@@ -589,7 +554,8 @@ glade_design_layout_button_press_event (GtkWidget *widget, GdkEventButton *ev)
   GtkWidget *child;
   gint x, y;
 
-  if (ev->button != 1 || ev->type != GDK_BUTTON_PRESS ||
+  if (ev->button != 1 ||
+      (ev->type != GDK_BUTTON_PRESS && ev->type != GDK_2BUTTON_PRESS) ||
       (child = gtk_bin_get_child (GTK_BIN (widget))) == NULL)
     return FALSE;
 
@@ -694,8 +660,9 @@ glade_design_layout_button_press_event (GtkWidget *widget, GdkEventButton *ev)
   priv->dx = x - (child_allocation.x + child_allocation.width + priv->child_offset);
   priv->dy = y - (child_allocation.y + child_allocation.height + priv->child_offset);
 
-  if (activity != ACTIVITY_NONE &&
-      !glade_project_is_toplevel_active (priv->project, child))
+  if (activity != ACTIVITY_NONE && 
+      (!glade_project_is_toplevel_active (priv->project, child) ||
+      ev->type == GDK_2BUTTON_PRESS))
     {
       _glade_design_view_freeze (priv->view);
       glade_project_selection_set (priv->project, G_OBJECT (child), TRUE);
@@ -704,7 +671,6 @@ glade_design_layout_button_press_event (GtkWidget *widget, GdkEventButton *ev)
 
   return FALSE;
 }
-
 static gboolean
 glade_design_layout_button_release_event (GtkWidget *widget,
                                           GdkEventButton *ev)
@@ -877,7 +843,11 @@ update_rectangles (GladeDesignLayoutPrivate *priv, GtkAllocation *alloc)
   priv->south.width = alloc->width;
   
   /* Update south east rectangle width */
-  pango_layout_get_pixel_size (priv->widget_name, &width, &height);
+  if (priv->widget_name)
+    pango_layout_get_pixel_size (priv->widget_name, &width, &height);
+  else
+    width = height = 0;
+
   priv->layout_width = width + (OUTLINE_WIDTH*2);
   width = MIN (alloc->width, width);
 
@@ -914,7 +884,7 @@ glade_design_layout_size_allocate (GtkWidget *widget,
       gint height, offset;
 
       offset = gtk_container_get_border_width (GTK_CONTAINER (widget)) + PADDING + OUTLINE_WIDTH;
-      priv->child_offset = offset;
+      priv->child_rect.x = priv->child_rect.y = priv->child_offset = offset;
 
       if (priv->widget_name)
         pango_layout_get_pixel_size (priv->widget_name, NULL, &height);
@@ -922,8 +892,8 @@ glade_design_layout_size_allocate (GtkWidget *widget,
         height = PADDING;
       
       alloc.x = alloc.y = 0;
-      priv->current_width = alloc.width = allocation->width - (offset * 2);
-      priv->current_height = alloc.height = allocation->height - (offset + OUTLINE_WIDTH * 1.5 + height);
+      priv->child_rect.width = alloc.width = allocation->width - (offset * 2);
+      priv->child_rect.height = alloc.height = allocation->height - (offset + OUTLINE_WIDTH * 1.5 + height);
       
       if (gtk_widget_get_realized (widget))
         gdk_window_move_resize (priv->offscreen_window,
@@ -934,13 +904,22 @@ glade_design_layout_size_allocate (GtkWidget *widget,
     }
 }
 
+static inline void
+update_widget_name (GladeDesignLayout *layout, GladeWidget *gwidget) 
+{
+  GladeDesignLayoutPrivate *priv = layout->priv;
+
+  if (priv->widget_name && gwidget)
+    {
+      pango_layout_set_text (priv->widget_name, glade_widget_get_name (gwidget), -1);
+      gtk_widget_queue_resize (GTK_WIDGET (layout));
+    }
+}
+
 static void
 on_glade_widget_name_notify (GObject *gobject, GParamSpec *pspec, GladeDesignLayout *layout) 
 {
-  GladeDesignLayoutPrivate *priv = layout->priv;
-  
-  pango_layout_set_text (priv->widget_name, glade_widget_get_name (GLADE_WIDGET (gobject)), -1);
-  gtk_widget_queue_resize (GTK_WIDGET (layout));
+  update_widget_name (layout, GLADE_WIDGET (gobject));
 }
 
 static void
@@ -948,20 +927,22 @@ glade_design_layout_add (GtkContainer *container, GtkWidget *widget)
 {
   GladeDesignLayout *layout = GLADE_DESIGN_LAYOUT (container);
   GladeDesignLayoutPrivate *priv = layout->priv;
-  GladeWidget *gchild;
 
-  layout->priv->current_width = 0;
-  layout->priv->current_height = 0;
+  priv->child_rect.width = 0;
+  priv->child_rect.height = 0;
 
   gtk_widget_set_parent_window (widget, priv->offscreen_window);
 
   GTK_CONTAINER_CLASS (glade_design_layout_parent_class)->add (container,
                                                                widget);
 
-  if ((gchild = glade_widget_get_from_gobject (G_OBJECT (widget))))
+  if (!priv->gchild &&
+      (priv->gchild = glade_widget_get_from_gobject (G_OBJECT (widget))))
     {
-      on_glade_widget_name_notify (G_OBJECT (gchild), NULL, layout);
-      g_signal_connect (gchild, "notify::name", G_CALLBACK (on_glade_widget_name_notify), layout);
+      update_widget_name (layout, priv->gchild);
+      g_signal_connect (priv->gchild, "notify::name",
+                        G_CALLBACK (on_glade_widget_name_notify),
+                        layout);
     }
     
   gtk_widget_queue_draw (GTK_WIDGET (container)); 
@@ -973,8 +954,14 @@ glade_design_layout_remove (GtkContainer *container, GtkWidget *widget)
   GladeWidget *gchild;
 
   if ((gchild = glade_widget_get_from_gobject (G_OBJECT (widget))))
-    g_signal_handlers_disconnect_by_func (gchild, on_glade_widget_name_notify,
-                                          GLADE_DESIGN_LAYOUT (container));
+    {
+      GladeDesignLayoutPrivate *priv = GLADE_DESIGN_LAYOUT_PRIVATE (container);
+      
+      g_signal_handlers_disconnect_by_func (gchild, on_glade_widget_name_notify,
+                                            GLADE_DESIGN_LAYOUT (container));
+      if (gchild == priv->gchild)
+        priv->gchild = NULL;
+    }
 
   GTK_CONTAINER_CLASS (glade_design_layout_parent_class)->remove (container, widget);
   gtk_widget_queue_draw (GTK_WIDGET (container));
@@ -1186,30 +1173,27 @@ draw_pixel_value (cairo_t *cr,
 {
   cairo_text_extents_t extents;
   gchar pixel_str[8];
-  gdouble xx, yy;
 
   g_snprintf (pixel_str, 8, "%d", val);
 
   cairo_text_extents (cr, pixel_str, &extents);
 
+  cairo_save (cr);
+  
   if (rotate)
     {
-      xx = x - 1.5;
-      yy = y + .5 + extents.width/2;
+      cairo_translate (cr, x - 1.5, y + .5 + extents.width/2);
       cairo_rotate (cr, G_PI/-2);
-      cairo_device_to_user (cr, &xx, &yy);
     }
   else
-    {
-      xx = x - (extents.width+extents.x_bearing)/2;
-      yy = y - 2;
-    }
+    cairo_translate (cr, x - (extents.width+extents.x_bearing)/2, y - 2);
 
+  cairo_move_to (cr, 0, 0);
+  
   if (draw_border || extents.width + 4 >= val)
     {
       cairo_set_source_rgba (cr, bg->red, bg->green, bg->blue, .9);
 
-      cairo_move_to (cr, xx, yy);
       cairo_text_path (cr, pixel_str);
       cairo_set_line_width (cr, 3);
       cairo_stroke (cr);
@@ -1218,10 +1202,9 @@ draw_pixel_value (cairo_t *cr,
       gdk_cairo_set_source_rgba (cr, fg);
     }
 
-  cairo_move_to (cr, xx, yy);
   cairo_show_text (cr, pixel_str);
 
-  if (rotate) cairo_rotate (cr, G_PI/2);
+  cairo_restore (cr);
 }
 
 static void
@@ -1290,11 +1273,6 @@ draw_dimensions (cairo_t *cr,
       if (right) draw_hmark (cr, x + w + right, yy);
 
       draw_stroke_lines (cr, bg, fg, top < DIMENSION_OFFSET+OUTLINE_WIDTH);
-
-      /* Draw pixel values */
-      draw_pixel_value (cr, bg, fg, x + w/2, yy, FALSE, h_clutter, w+1);
-      if (left) draw_pixel_value (cr,bg, fg, x - left/2, yy, FALSE, h_clutter, left);
-      if (right) draw_pixel_value (cr,bg, fg, x + w + right/2, yy, FALSE, h_clutter, right);
     }
   
   if (top || bottom)
@@ -1321,7 +1299,18 @@ draw_dimensions (cairo_t *cr,
       if (bottom) draw_vmark (cr, xx, y + h + bottom);
 
       draw_stroke_lines (cr, bg, fg, v_clutter);
+    }
 
+  if (left || right)
+    {
+      /* Draw pixel values */
+      draw_pixel_value (cr, bg, fg, x + w/2, yy, FALSE, h_clutter, w+1);
+      if (left) draw_pixel_value (cr,bg, fg, x - left/2, yy, FALSE, h_clutter, left);
+      if (right) draw_pixel_value (cr,bg, fg, x + w + right/2, yy, FALSE, h_clutter, right);
+    }
+  
+  if (top || bottom)
+    {
       /* Draw pixel values */
       draw_pixel_value (cr,bg, fg, xx, y + h/2, TRUE, v_clutter, h+1);
       if (top) draw_pixel_value (cr,bg, fg, xx, y - top/2, TRUE, v_clutter, top);
@@ -1339,12 +1328,11 @@ draw_pushpin (cairo_t *cr, gdouble x, gdouble y, gint angle,
   if (active)
     {
       outline = outline2;
-      x += .5;
+      cairo_translate (cr, x + .5, y);
       cairo_rotate (cr, angle*(G_PI/180));
-      cairo_device_to_user (cr, &x, &y);
     }
   else
-    x += 1.5;
+    cairo_translate (cr, x + 1.5, y);
 
   /* Swap colors if mouse is over */
   if (over)
@@ -1353,9 +1341,8 @@ draw_pushpin (cairo_t *cr, gdouble x, gdouble y, gint angle,
       outline = fill;
       fill = tmp;
     }
-  
-  cairo_translate (cr, x, y);
 
+  cairo_move_to (cr, 0, 0);
   _glade_design_layout_draw_pushpin (cr, (active) ? 2.5 : 4, outline, fill,
                                      (over) ? outline : fill, fg);
   
@@ -1372,25 +1359,23 @@ draw_selection_nodes (cairo_t *cr,
   GtkWidget *widget = priv->selection;
   gint top, bottom, left, right;
   gint x1, x2, x3, y1, y2, y3;
-  GtkAllocation alloc, palloc;
+  GtkAllocation alloc;
   GdkRGBA *c1, *c2, *c3, *fg;
   gint x, y, w, h;
+  gint offset = priv->child_offset;
+  GdkRectangle clip = {0, };
 
-  gtk_widget_get_allocation (widget, &alloc);
-  if (alloc.x < 0 || alloc.y < 0) return;
+  gtk_widget_translate_coordinates (widget, parent, 0, 0, &x, &y);
 
   c1 = &priv->frame_color_active[0];
   c2 = &priv->frame_color_active[1];
   c3 = &priv->frame_color[0];
   fg = &priv->fg_color;
   
-  gtk_widget_get_allocation (parent, &palloc);
-  
+  gtk_widget_get_allocation (widget, &alloc);
   w = alloc.width;
   h = alloc.height;
   
-  gtk_widget_translate_coordinates (widget, parent, 0, 0, &x, &y);
-
   top = gtk_widget_get_margin_top (widget);
   bottom = gtk_widget_get_margin_bottom (widget);
   left = gtk_widget_get_margin_left (widget);
@@ -1404,6 +1389,39 @@ draw_selection_nodes (cairo_t *cr,
   y2 = y + h/2;
   y3 = y + h + bottom;
 
+  /* Calculate clipping region to avoid drawing nodes outside of the toplevel
+   * allocation. This could happen for example with GtkOverlay overlay children.
+   */
+  if (offset > x1)
+    {
+      clip.x = offset;
+      clip.width = 0;
+    }
+  else
+    {
+      clip.x = 0;
+      clip.width = offset;      
+    }
+  if (offset > y1)
+    {
+      clip.y = offset;
+      clip.height = 0;
+    }
+  else
+    {
+      clip.y = 0;
+      clip.height = offset;      
+    }
+
+  clip.width += (priv->child_rect.width + offset < x3) ?
+                 priv->child_rect.width : gdk_window_get_width (priv->window);
+  clip.height += (priv->child_rect.height + offset < y3) ?
+                  priv->child_rect.height : gdk_window_get_height (priv->window);
+
+  cairo_save (cr);
+  gdk_cairo_rectangle (cr, &clip);
+  cairo_clip (cr);
+
   /* Draw nodes */
   cairo_set_line_width (cr, OUTLINE_WIDTH);
 
@@ -1450,6 +1468,32 @@ draw_selection_nodes (cairo_t *cr,
           draw_pushpin (cr, x3, y2, 45, c3, c2, c1, fg, node & MARGIN_RIGHT, halign == GTK_ALIGN_END);
         }
     }
+
+  cairo_restore (cr);
+}
+
+static inline void
+draw_drag_dest (GladeDesignLayout *layout, cairo_t *cr, GladeWidget *drag)
+{
+  GladeDesignLayoutPrivate *priv = layout->priv;
+  GObject *obj = glade_widget_get_object (drag);
+  const double dashes = 3;
+  GtkWidget *widget;
+  gint x, y;
+
+  if (!GTK_IS_WIDGET (obj))
+    return;
+
+  widget = GTK_WIDGET (obj);
+
+  gtk_widget_translate_coordinates (widget, GTK_WIDGET (layout), 0, 0, &x, &y);
+  cairo_set_line_width (cr, 2);
+  cairo_set_dash (cr, &dashes, 1, 0);
+  gdk_cairo_set_source_rgba (cr, &priv->frame_color_active[0]);
+  cairo_rectangle (cr, x+1, y+1, 
+                   gtk_widget_get_allocated_width (widget)-2,
+                   gtk_widget_get_allocated_height(widget)-2);
+  cairo_stroke (cr);
 }
 
 static gboolean
@@ -1472,8 +1516,7 @@ glade_design_layout_draw (GtkWidget *widget, cairo_t *cr)
           /* draw offscreen widgets */
           gdk_cairo_set_source_window (cr, priv->offscreen_window,
                                        priv->child_offset, priv->child_offset);
-          cairo_rectangle (cr, priv->child_offset, priv->child_offset,
-                           priv->current_width, priv->current_height);
+          gdk_cairo_rectangle (cr, &priv->child_rect);
           cairo_fill (cr);
 
           /* Draw selection */
@@ -1490,7 +1533,17 @@ glade_design_layout_draw (GtkWidget *widget, cairo_t *cr)
                   if (GTK_IS_WIDGET (selection) && 
                       gtk_widget_is_ancestor (selection, child))
                   {
+                    GdkRectangle *rect = &priv->child_rect;
+                    cairo_save (cr);
+                    cairo_rectangle (cr,
+                                     rect->x - OUTLINE_WIDTH/2,
+                                     rect->y - OUTLINE_WIDTH/2, 
+                                     rect->width + OUTLINE_WIDTH,
+                                     rect->height + OUTLINE_WIDTH);
+                    cairo_clip (cr);
                     draw_selection (cr, widget, selection, &priv->frame_color_active[0]);
+                    cairo_restore (cr);
+                    
                     selected = TRUE;
                   }
                 }
@@ -1502,12 +1555,15 @@ glade_design_layout_draw (GtkWidget *widget, cairo_t *cr)
           draw_frame (cr, priv, selected,
                       border_width + PADDING,
                       border_width + PADDING,
-                      priv->current_width + 2 * OUTLINE_WIDTH,
-                      priv->current_height + 2 * OUTLINE_WIDTH);
+                      priv->child_rect.width + 2 * OUTLINE_WIDTH,
+                      priv->child_rect.height + 2 * OUTLINE_WIDTH);
 
           /* Draw selection nodes if we are in margins edit mode */
-          if (priv->selection)
+          if (priv->selection && gtk_widget_is_ancestor (priv->selection, child))
             draw_selection_nodes (cr, priv, widget);
+
+          if (priv->drag_dest)
+            draw_drag_dest (GLADE_DESIGN_LAYOUT (widget), cr, priv->drag_dest);
         }
     }
   else if (gtk_cairo_should_draw_window (cr, priv->offscreen_window))
@@ -1596,12 +1652,12 @@ offscreen_window_from_parent (GdkWindow     *window,
   to_child (bin, parent_x, parent_y, offscreen_x, offscreen_y);
 }
 
-
 static void
-glade_design_layout_realize (GtkWidget * widget)
+glade_design_layout_realize (GtkWidget *widget)
 {
   GladeDesignLayoutPrivate *priv;
   GdkWindowAttr attributes;
+  GtkWidget *child;
   gint attributes_mask, border_width;
   GtkAllocation allocation;
   GdkDisplay *display;
@@ -1641,14 +1697,26 @@ glade_design_layout_realize (GtkWidget * widget)
                     G_CALLBACK (pick_offscreen_child), widget);
 
   /* Offscreen window */
+  child = gtk_bin_get_child (GTK_BIN (widget));
   attributes.window_type = GDK_WINDOW_OFFSCREEN;
   attributes.x = attributes.y = 0;
-  attributes.width = attributes.height = 0;
+
+  if (child && gtk_widget_get_visible (child))
+    {
+      GtkAllocation alloc;
+
+      gtk_widget_get_allocation (child, &alloc);
+      attributes.width = alloc.width;
+      attributes.height = alloc.height;
+    }
+    else
+      attributes.width = attributes.height = 0;
 
   priv->offscreen_window = gdk_window_new (gtk_widget_get_root_window (widget),
                                            &attributes, attributes_mask);
   gdk_window_set_user_data (priv->offscreen_window, widget);
-  
+
+  if (child) gtk_widget_set_parent_window (child, priv->offscreen_window);
   gdk_offscreen_window_set_embedder (priv->offscreen_window, priv->window);
 
   g_signal_connect (priv->offscreen_window, "to-embedder",
@@ -1675,6 +1743,9 @@ glade_design_layout_realize (GtkWidget * widget)
   priv->cursors[ACTIVITY_MARGINS_BOTTOM_RIGHT] = g_object_ref (priv->cursors[ACTIVITY_RESIZE_WIDTH_AND_HEIGHT]);
   
   priv->widget_name = pango_layout_new (gtk_widget_get_pango_context (widget));
+  if (child)
+    update_widget_name (GLADE_DESIGN_LAYOUT (widget),
+                        glade_widget_get_from_gobject (child));
 }
 
 static void
@@ -1890,7 +1961,8 @@ glade_design_layout_finalize (GObject *object)
   GladeDesignLayoutPrivate *priv = layout->priv;
 
   g_clear_object (&priv->default_context);
-
+  g_clear_object (&priv->drag_dest);
+  
   g_signal_handlers_disconnect_by_func (priv->project,
                                         on_project_selection_changed,
                                         layout);
@@ -1901,109 +1973,25 @@ glade_design_layout_finalize (GObject *object)
   G_OBJECT_CLASS (glade_design_layout_parent_class)->finalize (object);
 }
 
-static gboolean
-on_drag_icon_draw (GtkWidget *widget, cairo_t *cr, GladeDesignLayout *layout)
-{
-  GtkStyleContext *context = gtk_widget_get_style_context (widget);
-  cairo_pattern_t *gradient;
-  GtkAllocation alloc;
-  gint x, y, w, h;
-  gdouble h2;
-  GdkRGBA bg;
-  
-  gtk_widget_get_allocation (widget, &alloc);
-  x = alloc.x;
-  y = alloc.y;
-  w = alloc.width;
-  h = alloc.height;
-  h2 = h/2.0;
-
-  gtk_style_context_get_background_color (context, GTK_STATE_NORMAL, &bg);
-
-  gradient = cairo_pattern_create_linear (x, y, x, y+h);
-  cairo_pattern_add_color_stop_rgba (gradient, 0, bg.red, bg.green, bg.blue, 0);
-  cairo_pattern_add_color_stop_rgba (gradient, .5, bg.red, bg.green, bg.blue, .8);
-  cairo_pattern_add_color_stop_rgba (gradient, 1, bg.red, bg.green, bg.blue, 0);
-
-  cairo_set_source (cr, gradient);
-  cairo_rectangle (cr, x+h2, y, w-h, h);
-  cairo_fill (cr);
-  cairo_pattern_destroy (gradient);
-
-  gradient = cairo_pattern_create_radial (x+h2, y+h2, 0, x+h2, y+h2, h2);
-  cairo_pattern_add_color_stop_rgba (gradient, 0, bg.red, bg.green, bg.blue, .8);
-  cairo_pattern_add_color_stop_rgba (gradient, 1, bg.red, bg.green, bg.blue, 0);
-
-  cairo_set_source (cr, gradient);
-  cairo_rectangle (cr, x, y, h2, h);
-  cairo_fill (cr);
-
-  cairo_translate (cr, w-h, 0);
-  cairo_set_source (cr, gradient);
-  cairo_rectangle (cr, x+h2, y, h2, h);
-  cairo_fill (cr);
-  
-  cairo_pattern_destroy (gradient);
-
-  return FALSE;
-}
-
-GtkWidget *
-_glade_design_layout_dnd_icon_widget_new (GdkDragContext *context,
-                                          const gchar *icon_name,
-                                          const gchar *description)
-{
-  GtkWidget *window, *box, *label, *icon;
-  GdkScreen *screen;
-  GdkVisual *visual;
-
-  screen = gdk_window_get_screen (gdk_drag_context_get_source_window (context));
-  visual = gdk_screen_get_rgba_visual (screen);
-  window = gtk_window_new (GTK_WINDOW_POPUP);
-  
-  gtk_window_set_type_hint (GTK_WINDOW (window), GDK_WINDOW_TYPE_HINT_DND);
-  gtk_window_set_screen (GTK_WINDOW (window), screen);
-  gtk_widget_set_visual (window, visual);
-
-  box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 4);
-  gtk_container_set_border_width (GTK_CONTAINER (box), 12);
-
-  icon = gtk_image_new_from_icon_name (icon_name, GTK_ICON_SIZE_BUTTON);
-  gtk_widget_set_opacity (icon, .8);
-  
-  label = gtk_label_new (description);
-
-  gtk_box_pack_start (GTK_BOX (box), icon, FALSE, TRUE, 0);
-  gtk_box_pack_start (GTK_BOX (box), label, FALSE, TRUE, 0);
-
-  gtk_widget_show_all (box);
-  gtk_container_add (GTK_CONTAINER (window), box);
-
-  gtk_widget_set_app_paintable (window, TRUE);
-  g_signal_connect (window, "draw", G_CALLBACK (on_drag_icon_draw), NULL);
-
-  return window;
-}
-
 static void
 glade_design_layout_drag_begin (GtkWidget *widget, GdkDragContext *context)
 {
   GladeDesignLayoutPrivate *priv = GLADE_DESIGN_LAYOUT_PRIVATE (widget);
   GladeWidgetAdaptor *adaptor;
   GladeWidget *gwidget;
-  const gchar *icon_name;
   gchar *description;
 
   gwidget = glade_widget_get_from_gobject (priv->drag_source);
   adaptor = glade_widget_get_adaptor (gwidget);
-  icon_name = glade_widget_adaptor_get_icon_name (adaptor);
   description = g_strdup_printf ("%s [%s]",
                                  glade_widget_adaptor_get_name (adaptor),
                                  glade_widget_get_name (gwidget));
 
-  priv->drag_icon = _glade_design_layout_dnd_icon_widget_new (context, icon_name, description);
-  g_object_ref_sink (priv->drag_icon);
-  gtk_drag_set_icon_widget (context, priv->drag_icon, 0, 0);
+  _glade_dnd_set_icon_widget (context,
+                              glade_widget_adaptor_get_icon_name (adaptor),
+                              description);
+  
+  g_free (description);
 }
 
 static void
@@ -2014,25 +2002,13 @@ glade_design_layout_drag_data_get (GtkWidget        *widget,
                                    guint             time)
 {
   GladeDesignLayoutPrivate *priv = GLADE_DESIGN_LAYOUT_PRIVATE (widget);
-
-  if (priv->drag_source)
-    {
-      static GdkAtom type = 0;
-
-      if (!type)
-        type = gdk_atom_intern_static_string (GDL_DND_TARGET_WIDGET);
-
-      gtk_selection_data_set (data, type, sizeof (gpointer),
-                              (const guchar *)&priv->drag_source, sizeof (gpointer));
-    }
+  _glade_dnd_set_data (data, G_OBJECT (priv->drag_source));
 }
 
 static void
 glade_design_layout_drag_end (GtkWidget *widget, GdkDragContext *context)
 {
   GladeDesignLayoutPrivate *priv = GLADE_DESIGN_LAYOUT_PRIVATE (widget);
-
-  g_clear_object (&priv->drag_icon);
   priv->drag_source = NULL;
 }
 
@@ -2221,13 +2197,6 @@ _glade_design_layout_get_colors (GtkStyleContext *context,
   c1->blue += off;
 }
 
-GtkTargetEntry *
-_glade_design_layout_get_dnd_target (void)
-{
-  static GtkTargetEntry target = {GDL_DND_TARGET_WIDGET, GTK_TARGET_SAME_APP, GDL_DND_INFO_WIDGET};
-  return &target;
-}
-
 void
 _glade_design_layout_get_hot_point (GladeDesignLayout *layout,
                                     gint *x,
@@ -2242,6 +2211,231 @@ _glade_design_layout_get_hot_point (GladeDesignLayout *layout,
     *y = priv->drag_y;
 }
 
+void
+_glade_design_layout_set_highlight (GladeDesignLayout *layout,
+                                    GladeWidget       *drag)
+{
+  GladeDesignLayoutPrivate *priv = layout->priv;
+
+  g_clear_object (&priv->drag_dest);
+
+  if (drag)
+    priv->drag_dest = g_object_ref (drag);
+
+  gtk_widget_queue_draw (GTK_WIDGET (layout));
+}
+
+typedef struct
+{
+  GtkWidget *toplevel;
+  gint x, y;
+  GtkWidget *child;
+  gint level;
+} FindInContainerData;
+
+static void
+find_first_child_inside_container (GtkWidget *widget, FindInContainerData *data)
+{
+  gint x, y, w, h;
+
+  if (data->child || !gtk_widget_get_mapped (widget))
+    return;
+
+  gtk_widget_translate_coordinates (data->toplevel, widget, data->x, data->y,
+                                    &x, &y);
+  
+  /* Margins are not part of the widget allocation */
+  w = gtk_widget_get_allocated_width (widget) + gtk_widget_get_margin_right (widget);
+  h = gtk_widget_get_allocated_height (widget) + gtk_widget_get_margin_bottom (widget);
+
+  if (x >= (0 - gtk_widget_get_margin_left (widget)) && x < w &&
+      y >= (0 - gtk_widget_get_margin_top (widget)) && y < h)
+    {
+      if (GLADE_IS_PLACEHOLDER (widget))
+        data->child = widget;
+      else
+        {
+          GladeWidget *gwidget = glade_widget_get_from_gobject (widget);
+
+          if (GTK_IS_CONTAINER (widget))
+            {
+              if (gwidget)
+                data->child = _glade_design_layout_get_child_at_position (widget, x, y);
+              else
+                gtk_container_forall (GTK_CONTAINER (widget),
+                                      (GtkCallback) find_first_child_inside_container,
+                                      data);
+            }
+          
+          if (!data->child && gwidget)
+            data->child = widget;
+        }
+    }
+}
+
+static void
+find_last_child_inside_container (GtkWidget *widget, FindInContainerData *data)
+{
+  gint x, y, w, h;
+
+  if ((data->child && data->level) || !gtk_widget_get_mapped (widget))
+    return;
+
+  gtk_widget_translate_coordinates (data->toplevel, widget, data->x, data->y,
+                                    &x, &y);
+
+  /* Margins are not part of the widget allocation */
+  w = gtk_widget_get_allocated_width (widget) + gtk_widget_get_margin_right (widget);
+  h = gtk_widget_get_allocated_height (widget) + gtk_widget_get_margin_bottom (widget);
+
+  if (x >= (0 - gtk_widget_get_margin_left (widget)) && x < w &&
+      y >= (0 - gtk_widget_get_margin_top (widget)) && y < h)
+    {
+      GladeWidget *gwidget = glade_widget_get_from_gobject (widget);
+
+      if (GTK_IS_CONTAINER (widget))
+        {
+          if (!data->level)
+            data->child = NULL;
+
+          data->level++;
+
+          if (gwidget)
+            data->child = _glade_design_layout_get_child_at_position (widget, x, y);
+          else
+            gtk_container_forall (GTK_CONTAINER (widget),
+                                  (GtkCallback) find_last_child_inside_container,
+                                  data);
+          data->level--;
+        }
+
+      if (data->level)
+        {
+          if (!data->child && (GLADE_IS_PLACEHOLDER (widget) || gwidget))
+            data->child = widget;
+        }
+      else if ((!data->child || data->toplevel == gtk_widget_get_parent (data->child)) &&
+               (GLADE_IS_PLACEHOLDER (widget) || gwidget))
+        data->child = widget;
+    }
+}
+
+GtkWidget *
+_glade_design_layout_get_child_at_position (GtkWidget *widget, gint x, gint y)
+{
+  gboolean find_last;
+
+  if (!gtk_widget_get_mapped (widget))
+    return NULL;
+
+  find_last = (GTK_IS_FIXED (widget) || GTK_IS_LAYOUT (widget) || GTK_IS_OVERLAY (widget));
+  
+  if (x >= 0 && x <= gtk_widget_get_allocated_width (widget) &&
+      y >= 0 && y <= gtk_widget_get_allocated_height (widget))
+    {
+      if (GTK_IS_CONTAINER (widget))
+        {
+          FindInContainerData data = {widget, x, y, NULL, 0};
+
+          if (find_last)
+            gtk_container_forall (GTK_CONTAINER (widget),
+                                  (GtkCallback)find_last_child_inside_container,
+                                  &data);
+          else
+            gtk_container_forall (GTK_CONTAINER (widget),
+                                  (GtkCallback)find_first_child_inside_container,
+                                  &data);
+
+          return (data.child) ? data.child : widget;
+        }
+      else
+        return widget;
+    }
+  
+  return NULL;
+}
+
+static inline gboolean
+gdl_get_child_from_event (GladeDesignLayout *layout,
+                          GdkEvent          *event,
+                          GladeWidget      **gwidget,
+                          GtkWidget        **placeholder,
+                          gint              *x,
+                          gint              *y)
+{
+  GladeDesignLayoutPrivate *priv = layout->priv;
+  GtkWidget *child;
+
+  if (!priv->gchild)
+    return TRUE;
+      
+  _glade_design_layout_coords_from_event (priv->window, event, x, y);
+
+  child = GTK_WIDGET (glade_widget_get_object (priv->gchild));
+  if ((child = _glade_design_layout_get_child_at_position (child,
+                                                          *x - priv->child_offset,
+                                                          *y - priv->child_offset)))
+    {
+      if (GLADE_IS_PLACEHOLDER (child))
+        {
+          *gwidget = glade_placeholder_get_parent (GLADE_PLACEHOLDER (child));
+          *placeholder = child;
+        }
+      else
+        {
+          *gwidget = glade_widget_get_from_gobject (child);
+          *placeholder = NULL;
+        }
+
+      return FALSE;
+    }
+
+  return TRUE;
+}
+
+static inline void
+gdl_drag_source_check (GladeDesignLayout *layout,
+                       GladePointerMode   mode,
+                       GdkEvent          *event,
+                       GladeWidget       *gwidget,
+                       gint               x,
+                       gint               y)
+{
+  GladeDesignLayoutPrivate *priv = layout->priv;
+
+  if (mode == GLADE_POINTER_SELECT && 
+      event->type == GDK_BUTTON_PRESS &&
+      event->button.button == 1)
+    {
+      GObject *source;
+
+      if (gwidget && (source = glade_widget_get_object (gwidget)) &&
+          !(event->button.state & GDK_SHIFT_MASK) &&
+          _glade_drag_can_drag (GLADE_DRAG (gwidget)))
+        {
+          priv->drag_source = GTK_WIDGET (source);
+
+          gtk_widget_translate_coordinates (GTK_WIDGET (layout),
+                                            priv->drag_source, x, y,
+                                            &priv->drag_x, &priv->drag_y);
+        }
+      else
+        {
+          priv->drag_source = NULL;
+        }
+    }
+  else if (event->type == GDK_BUTTON_RELEASE && event->button.button == 1)
+    {
+      priv->drag_source = NULL;
+    }
+}
+
+GladeWidget *
+_glade_design_layout_get_child (GladeDesignLayout *layout)
+{
+  return layout->priv->gchild;
+}
+
 /*
  * _glade_design_layout_do_event:
  * @layout: A #GladeDesignLayout
@@ -2255,19 +2449,19 @@ _glade_design_layout_get_hot_point (GladeDesignLayout *layout,
 gboolean
 _glade_design_layout_do_event (GladeDesignLayout *layout, GdkEvent *event)
 {
+  GladeDesignLayoutPrivate *priv = layout->priv;
   GtkWidget *widget = GTK_WIDGET (layout);
-  GladeFindInContainerData data = { widget, 0, };
-  GladeDesignLayoutPrivate *priv;
+  GtkWidget *placeholder;
+  GladeWidget *gwidget;
   GladePointerMode mode;
   gboolean retval;
+  gint x, y;
   GList *l;
-
-  priv = layout->priv;
-
-  _glade_design_layout_coords_from_event (priv->window, event, &data.x, &data.y);
-
+  
+  if (gdl_get_child_from_event (layout, event, &gwidget, &placeholder, &x, &y))
+    return FALSE;
+    
   mode = glade_project_get_pointer_mode (priv->project);
-  glade_design_layout_find_inside_container (widget, &data);
  
   /* Check if we want to enter in margin edit mode */
   if ((event->type == GDK_BUTTON_PRESS || event->type == GDK_2BUTTON_PRESS)  &&
@@ -2277,7 +2471,7 @@ _glade_design_layout_do_event (GladeDesignLayout *layout, GdkEvent *event)
       g_list_next (l) == NULL && GTK_IS_WIDGET (l->data) && 
       gtk_widget_is_ancestor (l->data, widget))
     {
-      if (gdl_get_margins_from_pointer (widget, l->data, data.x, data.y))
+      if (gdl_get_margins_from_pointer (layout, l->data, x, y))
         {
           if (event->button.button == 2)
             {
@@ -2298,32 +2492,18 @@ _glade_design_layout_do_event (GladeDesignLayout *layout, GdkEvent *event)
         }
     }
 
-  if (event->type == GDK_BUTTON_PRESS && event->button.button == 1 &&
-      mode == GLADE_POINTER_SELECT)
-    {
-      GObject *source;
-
-      if (event->button.state & GDK_SHIFT_MASK)
-        {
-          priv->drag_source = NULL;
-        }
-      else if (data.gwidget && (source = glade_widget_get_object (data.gwidget)))
-        {
-          priv->drag_source = GTK_WIDGET (source);
-
-          gtk_widget_translate_coordinates (widget, priv->drag_source,
-                                            data.x, data.y,
-                                            &priv->drag_x, &priv->drag_y);
-        }
-    }
+  /* Check if this event could start a drag event and save the initial
+   * coordinates for later.
+   */
+  gdl_drag_source_check (layout, mode, event, gwidget, x, y);
 
   _glade_design_view_freeze (priv->view);
   
   /* Try the placeholder first */
-  if (data.placeholder && gtk_widget_event (data.placeholder, event)) 
+  if (placeholder && gtk_widget_event (placeholder, event)) 
     retval = TRUE;
-  else if (data.gwidget) /* Then we try a GladeWidget */
-    retval = glade_widget_event (data.gwidget, event);
+  else if (gwidget) /* Then we try a GladeWidget */
+    retval = glade_widget_event (gwidget, event);
   else
     retval = FALSE;
 
