@@ -67,10 +67,6 @@ enum
   N_PROPERTIES
 };
 
-#define GLADE_EDITOR_GET_PRIVATE(object) (G_TYPE_INSTANCE_GET_PRIVATE ((object),  \
-					  GLADE_TYPE_EDITOR,                      \
-					  GladeEditorPrivate))
-
 #define GLADE_EDITOR_PRIVATE(object) (((GladeEditor*)object)->priv)
 
 struct _GladeEditorPrivate
@@ -141,7 +137,7 @@ struct _GladeEditorPrivate
   gboolean show_class_field; /* Whether or not to show the class field at the top */
 };
 
-G_DEFINE_TYPE (GladeEditor, glade_editor, GTK_TYPE_BOX);
+G_DEFINE_TYPE_WITH_PRIVATE (GladeEditor, glade_editor, GTK_TYPE_BOX);
 
 static GParamSpec *properties[N_PROPERTIES];
 
@@ -262,20 +258,18 @@ glade_editor_class_init (GladeEditorClass *klass)
   /* Bind to template */
   gtk_widget_class_set_template_from_resource (widget_class, "/org/gnome/gladeui/glade-editor.ui");
 
-  gtk_widget_class_bind_child (widget_class, GladeEditorPrivate, notebook);
-  gtk_widget_class_bind_child (widget_class, GladeEditorPrivate, page_widget);
-  gtk_widget_class_bind_child (widget_class, GladeEditorPrivate, page_packing);
-  gtk_widget_class_bind_child (widget_class, GladeEditorPrivate, page_common);
-  gtk_widget_class_bind_child (widget_class, GladeEditorPrivate, page_atk);
-  gtk_widget_class_bind_child (widget_class, GladeEditorPrivate, class_field);
-  gtk_widget_class_bind_child (widget_class, GladeEditorPrivate, class_icon);
-  gtk_widget_class_bind_child (widget_class, GladeEditorPrivate, class_label);
-  gtk_widget_class_bind_child (widget_class, GladeEditorPrivate, warning);
-  gtk_widget_class_bind_child (widget_class, GladeEditorPrivate, signal_editor);
+  gtk_widget_class_bind_template_child_private (widget_class, GladeEditor, notebook);
+  gtk_widget_class_bind_template_child_private (widget_class, GladeEditor, page_widget);
+  gtk_widget_class_bind_template_child_private (widget_class, GladeEditor, page_packing);
+  gtk_widget_class_bind_template_child_private (widget_class, GladeEditor, page_common);
+  gtk_widget_class_bind_template_child_private (widget_class, GladeEditor, page_atk);
+  gtk_widget_class_bind_template_child_private (widget_class, GladeEditor, class_field);
+  gtk_widget_class_bind_template_child_private (widget_class, GladeEditor, class_icon);
+  gtk_widget_class_bind_template_child_private (widget_class, GladeEditor, class_label);
+  gtk_widget_class_bind_template_child_private (widget_class, GladeEditor, warning);
+  gtk_widget_class_bind_template_child_private (widget_class, GladeEditor, signal_editor);
 
-  gtk_widget_class_bind_callback (widget_class, glade_editor_switch_page);
-
-  g_type_class_add_private (klass, sizeof (GladeEditorPrivate));
+  gtk_widget_class_bind_template_callback (widget_class, glade_editor_switch_page);
 }
 
 static void
@@ -373,7 +367,9 @@ glade_editor_init (GladeEditor *editor)
   GladeEditorPrivate *priv;
   gint                icon_height;
 
-  editor->priv = priv = GLADE_EDITOR_GET_PRIVATE (editor);
+  editor->priv = priv = glade_editor_get_instance_private (editor);
+
+  priv->show_class_field = TRUE;
 
   gtk_widget_init_template (GTK_WIDGET (editor));
 
