@@ -96,7 +96,7 @@ enum
   PROP_PROJECT,
 };
 
-G_DEFINE_TYPE (GladeProjectProperties, glade_project_properties, GTK_TYPE_DIALOG);
+G_DEFINE_TYPE_WITH_PRIVATE (GladeProjectProperties, glade_project_properties, GTK_TYPE_DIALOG);
 
 /********************************************************
  *                  Class/Instance Init                 *
@@ -106,10 +106,7 @@ glade_project_properties_init (GladeProjectProperties *properties)
 {
   GladeProjectPropertiesPrivate *priv;
 
-  properties->priv = priv =
-    G_TYPE_INSTANCE_GET_PRIVATE (properties,
-				 GLADE_TYPE_PROJECT_PROPERTIES,
-				 GladeProjectPropertiesPrivate);
+  properties->priv = priv = glade_project_properties_get_instance_private (properties);
 
   priv->target_radios = g_hash_table_new_full (g_str_hash, g_str_equal,
                                                g_free, NULL);
@@ -142,30 +139,28 @@ glade_project_properties_class_init (GladeProjectPropertiesClass *klass)
 
   /* Define the relationship of the private entry and the entry defined in the xml
    */
-  gtk_widget_class_bind_child (widget_class, GladeProjectPropertiesPrivate, resource_default_radio);
-  gtk_widget_class_bind_child (widget_class, GladeProjectPropertiesPrivate, resource_relative_radio);
-  gtk_widget_class_bind_child (widget_class, GladeProjectPropertiesPrivate, resource_fullpath_radio);
-  gtk_widget_class_bind_child (widget_class, GladeProjectPropertiesPrivate, relative_path_entry);
-  gtk_widget_class_bind_child (widget_class, GladeProjectPropertiesPrivate, full_path_button);
-  gtk_widget_class_bind_child (widget_class, GladeProjectPropertiesPrivate, domain_entry);
-  gtk_widget_class_bind_child (widget_class, GladeProjectPropertiesPrivate, template_checkbutton);
-  gtk_widget_class_bind_child (widget_class, GladeProjectPropertiesPrivate, template_combobox);
-  gtk_widget_class_bind_child (widget_class, GladeProjectPropertiesPrivate, toolkit_box);
+  gtk_widget_class_bind_template_child_private (widget_class, GladeProjectProperties, resource_default_radio);
+  gtk_widget_class_bind_template_child_private (widget_class, GladeProjectProperties, resource_relative_radio);
+  gtk_widget_class_bind_template_child_private (widget_class, GladeProjectProperties, resource_fullpath_radio);
+  gtk_widget_class_bind_template_child_private (widget_class, GladeProjectProperties, relative_path_entry);
+  gtk_widget_class_bind_template_child_private (widget_class, GladeProjectProperties, full_path_button);
+  gtk_widget_class_bind_template_child_private (widget_class, GladeProjectProperties, domain_entry);
+  gtk_widget_class_bind_template_child_private (widget_class, GladeProjectProperties, template_checkbutton);
+  gtk_widget_class_bind_template_child_private (widget_class, GladeProjectProperties, template_combobox);
+  gtk_widget_class_bind_template_child_private (widget_class, GladeProjectProperties, toolkit_box);
 
   /* Declare the callback ports that this widget class exposes, to bind with <signal>
    * connections defined in the GtkBuilder xml
    */
-  gtk_widget_class_bind_callback (widget_class, on_template_combo_box_changed);
-  gtk_widget_class_bind_callback (widget_class, on_template_checkbutton_toggled);
-  gtk_widget_class_bind_callback (widget_class, resource_default_toggled);
-  gtk_widget_class_bind_callback (widget_class, resource_relative_toggled);
-  gtk_widget_class_bind_callback (widget_class, resource_fullpath_toggled);
-  gtk_widget_class_bind_callback (widget_class, resource_path_activated);
-  gtk_widget_class_bind_callback (widget_class, resource_full_path_set);
-  gtk_widget_class_bind_callback (widget_class, verify_clicked);
-  gtk_widget_class_bind_callback (widget_class, on_domain_entry_changed);
-
-  g_type_class_add_private (gobject_class, sizeof (GladeProjectPropertiesPrivate));
+  gtk_widget_class_bind_template_callback (widget_class, on_template_combo_box_changed);
+  gtk_widget_class_bind_template_callback (widget_class, on_template_checkbutton_toggled);
+  gtk_widget_class_bind_template_callback (widget_class, resource_default_toggled);
+  gtk_widget_class_bind_template_callback (widget_class, resource_relative_toggled);
+  gtk_widget_class_bind_template_callback (widget_class, resource_fullpath_toggled);
+  gtk_widget_class_bind_template_callback (widget_class, resource_path_activated);
+  gtk_widget_class_bind_template_callback (widget_class, resource_full_path_set);
+  gtk_widget_class_bind_template_callback (widget_class, verify_clicked);
+  gtk_widget_class_bind_template_callback (widget_class, on_domain_entry_changed);
 }
 
 /********************************************************
