@@ -134,7 +134,9 @@ glade_util_get_type_from_name (const gchar *name, gboolean have_func)
         {
           g_warning (_("We could not find the symbol \"%s\""), func_name);
         }
-      g_free (func_name);
+
+      if (!have_func)
+	g_free (func_name);
     }
 
   if (type == 0)
@@ -261,6 +263,9 @@ glade_util_ui_message (GtkWidget *parent,
                         (gtk_dialog_get_content_area (GTK_DIALOG (dialog))),
                         widget, TRUE, TRUE, 2);
       gtk_widget_show (widget);
+
+      /* If theres additional content, make it resizable */
+      gtk_window_set_resizable (GTK_WINDOW (dialog), TRUE);
     }
   
   response = gtk_dialog_run (GTK_DIALOG (dialog));
@@ -898,7 +903,7 @@ glade_util_load_library (const gchar *library_name)
         }
     }
 
-  if (!module)
+  if (g_getenv (GLADE_ENV_TESTING) == NULL && !module)
     {
       const gchar *paths[] = { glade_app_get_modules_dir (),
                                glade_app_get_lib_dir (),
@@ -1642,7 +1647,6 @@ glade_utils_hijack_key_press (GtkWindow *win,
 }
 
 
-
 void
 glade_utils_cairo_draw_line (cairo_t *cr,
                              GdkColor *color,
@@ -1916,3 +1920,65 @@ glade_utils_pointer_mode_render_icon (GladePointerMode mode, GtkIconSize size)
 
   return pix;
 }
+
+/**
+ * glade_utils_get_pointer:
+ * @widget: The widget to get the mouse position relative for
+ * @window: The window of the current event, or %NULL
+ * @device: The device, if not specified, the current event will be expected to have a @device.
+ * @x: The location to store the mouse pointer X position
+ * @y: The location to store the mouse pointer Y position
+ *
+ * Get's the pointer position relative to @widget, while @window and @device
+ * are not absolutely needed, they should be passed wherever possible.
+ *
+ */
+void
+glade_utils_get_pointer (GtkWidget *widget,
+			 GdkWindow *window,
+			 GdkDevice *device,
+			 gint      *x,
+			 gint      *y)
+{
+  gint device_x = 0, device_y = 0;
+  gint final_x = 0, final_y = 0;
+  GtkWidget *event_widget = NULL;
+
+  g_return_if_fail (GTK_IS_WIDGET (widget));
+
+  if (!device)
+    {
+      GdkEvent *event = gtk_get_current_event ();
+
+      device = gdk_event_get_device (event);
+      gdk_event_free (event);
+    }
+
+  g_return_if_fail (GDK_IS_DEVICE (device));
+
+  if (!window)
+    window = gtk_widget_get_window (widget);
+
+  g_return_if_fail (GDK_IS_WINDOW (window));
+
+  gdk_window_get_device_position (window, device, &device_x, &device_y, NULL);
+  gdk_window_get_user_data (window, (gpointer)&event_widget);
+
+  if (event_widget != widget)
+    {
+      gtk_widget_translate_coordinates (event_widget,
+                                        widget,
+                                        device_x, device_y,
+					&final_x, &final_y);
+    }
+  else
+    {
+      final_x = device_x;
+      final_y = device_y;
+    }
+
+  if (x)
+    *x = final_x;
+  if (y)
+    *y = final_y;
+}
