@@ -215,6 +215,8 @@ gchar                  *glade_widget_generate_path_name     (GladeWidget      *w
 
 gboolean                glade_widget_is_ancestor            (GladeWidget      *widget,
 							     GladeWidget      *ancestor);
+gboolean                glade_widget_depends                (GladeWidget      *widget,
+							     GladeWidget      *other);
 
 GdkDevice              *glade_widget_get_device_from_event  (GdkEvent *event);
 
@@ -337,6 +339,11 @@ void                    glade_widget_set_name		    (GladeWidget      *widget,
  
 G_CONST_RETURN gchar   *glade_widget_get_name               (GladeWidget      *widget);
 
+void                    glade_widget_set_is_composite       (GladeWidget      *widget,
+							     gboolean          composite);
+
+gboolean                glade_widget_get_is_composite       (GladeWidget      *widget);
+
 void                    glade_widget_set_internal	    (GladeWidget      *widget,
 							     const gchar      *internal);
  
@@ -370,6 +377,7 @@ void                    glade_widget_push_superuser         (void);
  
 void                    glade_widget_pop_superuser          (void);
 
+void                    glade_widget_verify                 (GladeWidget      *widget);
 void                    glade_widget_set_support_warning    (GladeWidget      *widget,
 							     const gchar      *warning);
 G_CONST_RETURN gchar   *glade_widget_support_warning        (GladeWidget      *widget);
