@@ -49,8 +49,7 @@ struct _GladeButtonEditorClass
 };
 
 GType            glade_button_editor_get_type (void) G_GNUC_CONST;
-GtkWidget       *glade_button_editor_new      (GladeWidgetAdaptor *adaptor,
-					       GladeEditable      *editable);
+GtkWidget       *glade_button_editor_new      (void);
 
 G_END_DECLS
 
