@@ -3,25 +3,24 @@
  *                 handle free-form child placement for containers such as 
  *                 GtkFixed and GtkLayout.
  *
- * Copyright (C) 2006 The GNOME Foundation.
+ * Copyright (C) 2006, 2013 Tristan Van Berkom.
  *
  * Author(s):
  *      Tristan Van Berkom <tvb@gnome.org>
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * This library is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as
+ * published by the Free Software Foundation; either version 2.1 of
+ * the License, or (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
- * USA.
+ * You should have received a copy of the GNU Lesser General Public 
+ * License along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -1255,3 +1254,13 @@ glade_fixed_get_type (void)
     }
   return fixed_type;
 }
+
+/* This is called from the catalog for a few widgets */
+GladeWidget *
+glade_gtk_create_fixed_widget (GladeWidgetAdaptor * adaptor,
+                               const gchar * first_property_name,
+                               va_list var_args)
+{
+  return (GladeWidget *) g_object_new_valist (GLADE_TYPE_FIXED,
+                                              first_property_name, var_args);
+}
