@@ -83,18 +83,6 @@ glade_gtk_widget_destroy_object (GladeWidgetAdaptor * adaptor,
   GWA_GET_CLASS (G_TYPE_OBJECT)->destroy_object (adaptor, object);
 }
 
-gboolean
-glade_gtk_widget_depends (GladeWidgetAdaptor * adaptor,
-                          GladeWidget * widget, GladeWidget * another)
-{
-  if (GTK_IS_ICON_FACTORY (glade_widget_get_object (another)) ||
-      GTK_IS_ACTION (glade_widget_get_object (another)) || 
-      GTK_IS_ACTION_GROUP (glade_widget_get_object (another)))
-    return TRUE;
-
-  return GWA_GET_CLASS (G_TYPE_OBJECT)->depends (adaptor, widget, another);
-}
-
 static void
 glade_gtk_parse_atk_relation (GladeProperty * property, GladeXmlNode * node)
 {
@@ -604,14 +592,25 @@ glade_gtk_widget_write_widget (GladeWidgetAdaptor * adaptor,
   if (prop && glade_property_get_enabled (prop))
     glade_property_write (prop, context, node);
 
-  /* First chain up and read in all the normal properties.. */
+  /* Finally chain up and read in all the normal properties.. */
   GWA_GET_CLASS (G_TYPE_OBJECT)->write_widget (adaptor, widget, context, node);
-
-  glade_gtk_write_accels (widget, context, node, TRUE);
-  glade_gtk_widget_write_atk_props (widget, context, node);
-  glade_gtk_widget_write_style_classes (widget, context, node);
 }
 
+void
+glade_gtk_widget_write_widget_after (GladeWidgetAdaptor * adaptor,
+				     GladeWidget * widget,
+				     GladeXmlContext * context, GladeXmlNode * node)
+{
+  /* The ATK properties are actually children */
+  glade_gtk_widget_write_atk_props (widget, context, node);
+
+  /* Put the accelerators and style classes after children */
+  glade_gtk_write_accels (widget, context, node, TRUE);
+  glade_gtk_widget_write_style_classes (widget, context, node);
+
+  /* Finally chain up and read in all the normal properties.. */
+  GWA_GET_CLASS (G_TYPE_OBJECT)->write_widget_after (adaptor, widget, context, node);
+}
 
 GladeEditorProperty *
 glade_gtk_widget_create_eprop (GladeWidgetAdaptor * adaptor,
@@ -865,31 +864,31 @@ glade_gtk_widget_action_activate (GladeWidgetAdaptor * adaptor,
         {
           GladeWidgetAdaptor *adaptor =
             glade_widget_adaptor_get_by_type (new_type);
-          GList *saved_props, *prop_cmds;
 	  GladeWidget *gnew_parent;
-          GladeProperty *property;
 
           glade_command_push_group (_("Adding parent %s for %s"),
                                     glade_widget_adaptor_get_title (adaptor), 
 				    glade_widget_get_name (gwidget));
 
-          /* Record packing properties */
-          saved_props =
-	    glade_widget_dup_properties (gwidget, glade_widget_get_packing_properties (gwidget),
-					 FALSE, FALSE, FALSE);
-
-
-	  property = glade_widget_get_parentless_widget_ref (gwidget);
-
-	  /* Remove "this" widget, If the parent we're removing is a parentless 
-	   * widget reference, the reference will be implicitly broken by the 'cut' command */
-          this_widget.data = gwidget;
-          glade_command_delete (&this_widget);
-
           /* Create new widget and put it where the placeholder was */
           if ((gnew_parent =
                glade_command_create (adaptor, gparent, NULL, project)) != NULL)
             {
+              GList *saved_props, *prop_cmds;
+              GladeProperty *property;
+
+              /* Record packing properties */
+              saved_props =
+                glade_widget_dup_properties (gwidget, glade_widget_get_packing_properties (gwidget),
+                                             FALSE, FALSE, FALSE);
+
+              /* Remove "this" widget, If the parent we're removing is a parentless
+               * widget reference, the reference will be implicitly broken by the 'cut' command */
+              this_widget.data = gwidget;
+              glade_command_delete (&this_widget);
+
+              property = glade_widget_get_parentless_widget_ref (gwidget);
+              
 	      /* Now we created the new parent, if gwidget had a parentless widget reference...
 	       * set that reference to the new parent instead */
 	      if (property)
@@ -921,15 +920,6 @@ glade_gtk_widget_action_activate (GladeWidgetAdaptor * adaptor,
               /* Add "this" widget to the new parent */
               glade_command_add (&this_widget, gnew_parent, NULL, project, FALSE);
             }
-          else
-	    {
-	      /* Create parent was cancelled, paste back to parent */
-	      glade_command_add (&this_widget, gparent, NULL, project, FALSE);
-
-	      /* Restore any parentless widget reference if there was one */
-	      if (property)
-		glade_command_set_property (property, glade_widget_get_object (gwidget));
-	    }
 
           glade_command_pop_group ();
         }
